#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import numpy as np
import copy
import pandas as pd
from docopt import docopt
from ._version import __version__
import emfile
from reliontomotools.fileIO import motlToRelion4, writeStarFile

__all__ = ['motl2RelionProgram']
# % Subtom motl file To Relion Converter

def motl2RelionProgram(args=None):
    doc = """motl2relion: converts subtom motl metadata to Relion.

    Usage:
      motl2relion.py (<em_file> | --evenodd <em_even_file> <em_odd_file>)\
      -o <star_file> -p=<angpix> [--bin=<binning>]\
      [-t=<label>] [--pad_zeros=<n_zeros>]

    Arguments:
                  <em_file>  EM file containing a subtom motl list.
             <em_even_file>  EM file containing half 1 of a motl list.
              <em_odd_file>  EM file containing half 2 of a motl list.
    Options:
                  --evenodd  Provides two half files instead of one single file.
       -p --angpix=<angpix>  Pixel size at bin 1 (in angstroms)
         -b --bin=<binning>  Input coordinates Bin factor in motl file [default: 1]
    -t --tomo_label=<label>  Prefix label for rlnTomoName [default: TS_]
      --pad_zeros=<n_zeros>  Number of padding zeros in tomogram number name.
                             Negative is auto [default: -1]

                 -h --help  Show this screen.
              -v --version  Show version.

    """

    arguments = docopt(doc, version=__version__)

    pixelSize = float(arguments['--angpix'])
    binning = int(arguments['--bin'])
    padZeros = int(arguments['--pad_zeros'])
    tomoLabel = arguments['--tomo_label']

    bothmotl = not arguments['<em_file>']
    if bothmotl:
        emFile1 = arguments['<em_even_file>']
        emFile2 = arguments['<em_odd_file>']
    else:
        emFile1 = arguments['<em_file>']

    fnStarOut = arguments['<star_file>']

    _, motl1 = emfile.read(emFile1)
    motl1 = motl1[0]
    dataStar1 = motlToRelion4(motl1, pixelSize,
                              binning=binning,
                              tomoLabel=tomoLabel,
                              padZeros=padZeros)

    if bothmotl:
        _, motl2 = emfile.read(emFile2)
        motl2 = motl2[0]
        dataStar2 = motlToRelion4(motl2, pixelSize,
                                  binning=binning,
                                  tomoLabel=tomoLabel,
                                  padZeros=padZeros)
        [dataStar1['data_particles']['_rlnRandomSubset']] = '1'
        [dataStar2['data_particles']['_rlnRandomSubset']] = '2'
        dataOut = copy.deepcopy(dataStar1)
        datap = pd.concat([dataStar1['data_particles'],
                           dataStar2['data_particles']])
        refIdx = np.array([int(partId) for partId in datap['_rlnTomoParticleId']])
        idxsort = np.argsort(refIdx)
        datap = datap.iloc[idxsort]

        dataOut['data_particles'] = datap
    else:
        dataOut = dataStar1

    writeStarFile(fnStarOut, dataOut)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import numpy as np
import copy
import pandas as pd
from docopt import docopt
from ._version import __version__
import emfile
from reliontomotools.fileIO import relion4ToMotl, readStarFile

__all__ = ['relion2motlProgram']
# % Subtom motl file To Relion Converter

def relion2motlProgram(args=None):
    doc = """relion2motl: converts relion starfile to subtom motl metadata.

    Usage:
      star_to_motl.py -i <star_file>  -o <em_file> [--bin=<binning>]\
       [--angpix=<angpix>] [--tomo_angles]
    
    Arguments:
               <star_file>  Relion star file containing data_optics and 
                            data_particles tables.
                 <em_file>  EM file containing converted orientations to subtom
                            motl list.
    
    Options:
           --bin=<binning>  Bin factor for output motl list [default: 1]
      -p --angpix=<angpix>  Pixel size at bin 1 (in angstroms). If negative,
                            pixel size is obtained from optics table [default: -1]

          -t --tomo_angles  Instead of rlnAngle{Rot,Tilt,AnglePsi}, angles from
                            rlnTomoSubtomogram{Rot,Tilt,AnglePsi} are used.
    
                 -h --help  Show this screen.
              -v --version  Show version.

    """

    arguments = docopt(doc, version=__version__)

    starFile = arguments['<star_file>']
    emFile = arguments['<em_file>']
    binning = int(arguments['--bin'])
    useTomoAngles = arguments['--tomo_angles']
    pixelSize = float(arguments['--angpix'])

    dataStar = readStarFile(starFile)

    print(pixelSize)
    if pixelSize < 0:
        if 'data_optics' in dataStar:
            print(pixelSize)
            datao = dataStar['data_optics']
            pixelSize = float(datao['_rlnTomoTiltSeriesPixelSize'])
        else:
            raise Exception(f"File {starFile} does not contain data_optics table "
                            "or rlnTomoTiltSeriesPixelSize parameter.")

    motl = relion4ToMotl(dataStar, pixelSize, binning, useTomoAngles)
    emfile.write(emFile, motl.astype(np.float32)[np.newaxis, ...], overwrite=True)



from setuptools import setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

pkg = 'reliontomotools'
exec(open(f'{pkg}/_version.py').read())

setup(
      name=pkg,
      version=__version__,
      author="Joaquín Otón",
      description="Additional tools for Relion tomo",
      long_description=long_description,
      long_description_content_type="text/markdown",
      url="https://github.com/joton/reliontomotools",
      project_urls={
        "Bug Tracker": "https://github.com/joton/reliontomotools/issues",
                    },
      classifiers=[
          "Programming Language :: Python :: 3",
          "License :: OSI Approved :: MIT License",
          "Operating System :: OS Independent",
                  ],
      packages=[pkg],
      license='MIT',
      entry_points={
          "console_scripts": [
              f"warptomo2relion = {pkg}.warptomo2relion:warpTomo2RelionProgram",
              f"motl2relion = {pkg}.motl2relion:motl2RelionProgram",
              f"relion2motl = {pkg}.relion2motl:relion2motlProgram",
          ]
                   },
      python_requires=">=3.7",
      install_requires=[
          'numpy',
          'pandas',
          'xmltodict',
          'docopt',
          'scipy',
          'tqdm',
          'transforms3d',
          'mrcfile',
          'emfile',
          ]

     )
